@@ -42,14 +42,22 @@ int ipc_helper_tcp_connection(void);
 int ipc_helper_closed_handle(void);
 int ipc_send_recv_helper(void);
 int ipc_helper_bind_twice(void);
+int ipc_helper_send_zero(void);
 int stdio_over_pipes_helper(void);
-int spawn_stdin_stdout(void);
+void spawn_stdin_stdout(void);
 int spawn_tcp_server_helper(void);
 
 static int maybe_run_test(int argc, char **argv);
 
 
 int main(int argc, char **argv) {
+#ifndef _WIN32
+  if (0 == geteuid() && NULL == getenv("UV_RUN_AS_ROOT")) {
+    fprintf(stderr, "The libuv test suite cannot be run as root.\n");
+    return EXIT_FAILURE;
+  }
+#endif
+
   if (platform_init(argc, argv))
     return EXIT_FAILURE;
 
@@ -66,7 +74,9 @@ int main(int argc, char **argv) {
     return EXIT_FAILURE;
   }
 
+#ifndef __SUNPRO_C
   return EXIT_SUCCESS;
+#endif
 }
 
 
@@ -104,6 +114,10 @@ static int maybe_run_test(int argc, char **argv) {
     return ipc_helper_bind_twice();
   }
 
+  if (strcmp(argv[1], "ipc_helper_send_zero") == 0) {
+    return ipc_helper_send_zero();
+  }
+
   if (strcmp(argv[1], "stdio_over_pipes_helper") == 0) {
     return stdio_over_pipes_helper();
   }
@@ -204,7 +218,8 @@ static int maybe_run_test(int argc, char **argv) {
 
   if (strcmp(argv[1], "spawn_helper9") == 0) {
     notify_parent_process();
-    return spawn_stdin_stdout();
+    spawn_stdin_stdout();
+    return 1;
   }
 
 #ifndef _WIN32
